@@ -7,6 +7,9 @@
 
 #include "Skanpage.h"
 
+#include <QFile>
+#include <QJsonDocument>
+#include <QJsonObject>
 #include <QThread>
 #include <QtQml>
 
@@ -21,6 +24,7 @@
 #include "DocumentSaver.h"
 #include "DocumentPrinter.h"
 #include "OCREngine.h"
+#include "ImageImport.h"
 #include "skanpage_debug.h"
 
 class SkanpagePrivate {
@@ -34,6 +38,7 @@ public:
     DocumentSaver m_documentSaver;
     DocumentPrinter m_documentPrinter;
     OCREngine m_OCREngine;
+    ImageImport m_imageImport;
     QThread m_fileIOThread;
     SkanpageConfiguration *m_configuration;
     KActionCollection *m_actionCollection;
@@ -54,11 +59,12 @@ public:
     QString m_deviceName;
     QString m_deviceVendor;
     QString m_deviceModel;
+    QUrl m_dumpOptionsUrl;
 };
 
 using namespace KSaneCore;
 
-Skanpage::Skanpage(const QString &deviceName, QObject *parent)
+Skanpage::Skanpage(const QString &deviceName, const QUrl &dumpOptionsUrl, QObject *parent)
     : QObject(parent)
     , d(std::make_unique<SkanpagePrivate>())
 {
@@ -84,6 +90,7 @@ Skanpage::Skanpage(const QString &deviceName, QObject *parent)
     d->m_documentSaver.moveToThread(&d->m_fileIOThread);
     d->m_OCREngine.moveToThread(&d->m_fileIOThread);
     d->m_documentSaver.setOCREngine(&d->m_OCREngine);
+    d->m_imageImport.moveToThread(&d->m_fileIOThread);
 
     connect(&d->m_documentHandler, &DocumentModel::saveDocument, &d->m_documentSaver, &DocumentSaver::saveDocument);
     connect(&d->m_documentHandler, &DocumentModel::saveNewPageTemporary, &d->m_documentSaver, &DocumentSaver::saveNewPageTemporary);
@@ -92,7 +99,10 @@ Skanpage::Skanpage(const QString &deviceName, QObject *parent)
     connect(&d->m_documentSaver, &DocumentSaver::fileSaved, &d->m_documentHandler, &DocumentModel::updateFileInformation);
     connect(&d->m_documentSaver, &DocumentSaver::sharingFileSaved, &d->m_documentHandler, &DocumentModel::updateSharingFileInformation);
     connect(&d->m_documentPrinter, &DocumentPrinter::showUserMessage, this, &Skanpage::showUserMessage);
+    connect(&d->m_imageImport, &ImageImport::showUserMessage, this, &Skanpage::showUserMessage);
+    connect(&d->m_imageImport, &ImageImport::imageImported, &d->m_documentHandler, &DocumentModel::addImage);
 
+    d->m_dumpOptionsUrl = dumpOptionsUrl;
     // try to open device from command line option first, then remembered device
     if (deviceName.isEmpty() || !openDevice(deviceName)) {
 
@@ -189,6 +199,10 @@ QRectF Skanpage::scanArea() const
 void Skanpage::setScanArea(QRectF area)
 {
     if (area == d->m_scanArea) return;
+    if (d->m_ksaneInterface.getOption(Interface::TopLeftXOption) == NULL ||
+        d->m_ksaneInterface.getOption(Interface::TopLeftYOption) == NULL ||
+        d->m_ksaneInterface.getOption(Interface::BottomRightXOption) == NULL ||
+        d->m_ksaneInterface.getOption(Interface::BottomRightYOption) == NULL) return;
     d->m_ksaneInterface.getOption(Interface::TopLeftXOption)->setValue(area.left() * d->m_maximumScanArea.width());
     d->m_ksaneInterface.getOption(Interface::TopLeftYOption)->setValue(area.top() * d->m_maximumScanArea.height());
     d->m_ksaneInterface.getOption(Interface::BottomRightXOption)->setValue(area.right() * d->m_maximumScanArea.width());
@@ -261,7 +275,12 @@ QImage Skanpage::previewImage() const
     return d->m_previewImage;
 }
 
-void Skanpage::preview()
+bool Skanpage::previewImageAvailable() const
+{
+    return !d->m_previewImage.isNull();
+}
+
+void Skanpage::previewScan()
 {
     if (Option *opt = d->m_ksaneInterface.getOption(Interface::TopLeftXOption)) opt->storeCurrentData();
     if (Option *opt = d->m_ksaneInterface.getOption(Interface::TopLeftYOption)) opt->storeCurrentData();
@@ -301,7 +320,9 @@ void Skanpage::startScan()
     if (!d->m_scanSubAreas.isEmpty()) {
         QRectF totalArea = d->m_scanArea; // Include last (unadded) area
         // This makes a rectangle that covers all the areas
-        for (const QRectF& area : d->m_scanSubAreas) totalArea = totalArea.united(area);
+        for (const QRectF& area : d->m_scanSubAreas) {
+            totalArea = totalArea.united(area);
+        }
         appendSubArea(d->m_scanArea); // Remember last area, for later use
         setScanArea(totalArea); // Scan all the needed area
     }
@@ -412,7 +433,26 @@ bool Skanpage::openDevice(const QString &deviceName, const QString &deviceVendor
         } else {
             showUserMessage(SkanpageUtils::ErrorMessage, QStringLiteral("Failed to open selected device."));
         }
-
+    }
+    if (!d->m_dumpOptionsUrl.isEmpty()) {
+        QJsonObject allScannerData;
+        allScannerData[QLatin1StringView("scanner")] = d->m_ksaneInterface.scannerDeviceToJson();
+        allScannerData[QLatin1StringView("options")] = d->m_ksaneInterface.scannerOptionsToJson();
+        QString absolutePath;
+        if (QDir::isAbsolutePath(d->m_dumpOptionsUrl.path())) {
+            absolutePath = d->m_dumpOptionsUrl.toLocalFile();
+        } else {
+            absolutePath = QDir::current().absoluteFilePath(d->m_dumpOptionsUrl.toLocalFile());
+        }
+        QJsonDocument jsonDocument(allScannerData);
+        QFile file(absolutePath);
+        if(!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
+            return false;
+        }
+        QTextStream out(&file);
+        out << jsonDocument.toJson();
+        qDebug() << QStringLiteral("The scanner device options have been saved to %1").arg(absolutePath);
+        file.close();
     }
     return status == Interface::OpeningSucceeded;
 }
@@ -439,6 +479,8 @@ void Skanpage::finishOpeningDevice(const QString &deviceName, const QString &dev
 
     d->m_scanAreaConnectionsDone = false;
     setupScanningBounds();
+    d->m_previewImage = QImage();
+    Q_EMIT previewImageChanged(d->m_previewImage);
 
     d->m_state = ReadyForScan;
     Q_EMIT applicationStateChanged(d->m_state);
@@ -644,4 +686,10 @@ void Skanpage::checkFinish()
     }
 }
 
+void Skanpage::importFile(const QUrl &fileUrl)
+{
+    const int currentDPI = d->m_ksaneInterface.getOption(KSaneCore::Interface::ResolutionOption)->value().toInt();
+    QMetaObject::invokeMethod(&d->m_fileIOThread, [this, fileUrl, currentDPI]() {d->m_imageImport.importImageFile(fileUrl, currentDPI);}, Qt::QueuedConnection);
+}
+
 #include "moc_Skanpage.cpp"
